#!/bin/bash

#
#you need to configure settings in /etc/gitlab/gitlab.rb
#gitlab_rails['backup_upload_connection'] = {
#  :provider => 'Local',
#  :local_root => '/mnt/backup01'
#}
#gitlab_rails['backup_upload_remote_directory'] = 'gitlab_backups'
#
#after this run:
#gitlab-ctl reconfigure
#
#for more information see:
#https://docs.gitlab.com/ce/raketasks/backup_restore.html#uploading-to-locally-mounted-shares

#backup directory
backup_directory="/mnt/backup01" #like a local_root in gitlab.rb
#subdirrectory of backup directory path. this option you set in gitlab.rb file to
backup_upload_remote_directory="gitlab_backups" # like a backup_upload_remote_directory in gitlab.rb
#configuration backup prefix
configuration_backup_prefix="etc-gitlab-"
#script working directory
script_working_directory="/root/scripts"
#log file name
log_file_name="gitlab_backup_tmp.log"

#remote storage path
remote_storage_path="set_up_remote_path"
#path to file with username and password to access remote storage
#
#minimize access permission with chmod 600 <file_path>
#
#file must contain:
#user=<username>
#pass=<password>
credential_file_path="/home/bissquit/gitlab_backup_credentials"
#default git uid
git_uid=998
#default git gid
git_gid=998

#time format for log messages
log_time_format="%d/%m/%Y %H:%M:%S"
#another storage already mount - true/false
already_mount=$(mount | grep ${backup_directory} | wc -l)
#backups store depth in days
configuration_backups_store_depth=7
#admin's e-mails
admin_email="set_up_admin_email" #may use comma to separate addresses

#execute command and check exit code
function execute_command {
        #execute command
        $2 &>>${script_working_directory}/${log_file_name}
        #check exit code
        if [ $?  != 0 ]
                then echo "[`date +"${log_time_format}]: "`$1 - Error, exit..." && echo "[`date +"%d/%m/%Y %H:%M:%S]: "`Full command for debug: $2" &>>${script_working_directory}/${log_file_name} ; exit
                else echo "[`date +"${log_time_format}]: "`$1 - Success" &>>${script_working_directory}/${log_file_name}
        fi
}

#check package status
function check_package {
        if [ 0 -eq "$(dpkg-query -W -f='${Status}' $1 2>/dev/null | grep -c "install ok installed" | wc -l)" ]
                then
                        echo "[`date +"${log_time_format}]: "`$1 has not installed. Installation starts after 10 second!" &>>${script_working_directory}/${log_file_name}
                        sleep 10
                        execute_command "[`date +"${log_time_format}]: "`Execute apt-get update" "apt-get update"
                        execute_command "[`date +"${log_time_format}]: "`$1 installation" "apt-get install -q -y $1"
                else echo "[`date +"${log_time_format}]: "`$1 has been installed" &>>${script_working_directory}/${log_file_name}
        fi
}

#----------------------script start--------------------------
#start
echo "[`date +"${log_time_format}]: "`Starting work" &>${script_working_directory}/${log_file_name}

#check cifs-utils
check_package "cifs-utils"

#check mailutils
check_package "mailutils"

#mount storage
if [ 1 -eq ${already_mount} ]
        then echo "[`date +"${log_time_format}]: "`Unknown storage already mount to ${backup_directory}. Dismounting..." &>>${script_working_directory}/${log_file_name} ; execute_command "Dismount unknown storage" "umount -l ${backup_directory}"
        else echo "[`date +"${log_time_format}]: "`Mount point ${backup_directory} is free" &>>${script_working_directory}/${log_file_name}
fi
execute_command "Mount remote storage" "mount.cifs ${remote_storage_path} ${backup_directory} -o credentials=${credential_file_path},uid=${git_uid},gid=${git_gid}"

#common info
echo "[`date +"${log_time_format}]: "`Gitlab version: $(aptitude show gitlab-ce | grep Версия | awk -F ": " '{print $2}')" &>>${script_working_directory}/${log_file_name}
echo "[`date +"${log_time_format}]: "`Backup directory: ${backup_directory}/${backup_upload_remote_directory}" &>>${script_working_directory}/${log_file_name}
echo "[`date +"${log_time_format}]: "`Backup directory contain $(ls -l ${backup_directory}/${backup_upload_remote_directory} | wc -l) files with a total size in $(du -h ${backup_directory}/${backup_upload_remote_directory} | awk '{ print $1 }')" &>>${script_working_directory}/${log_file_name}

#setting up low permissions
execute_command "Setting up permission mask" "umask 0077"

#backup gitlab configuration
execute_command "Backup gitlab configuration" "tar -cf ${backup_directory}/${backup_upload_remote_directory}/$(date "+${configuration_backup_prefix}%Y%m%d-%H%M%S.tar") -C / etc/gitlab"

#count of configuration backups
configuration_backups_count=$(find ${backup_directory}/${backup_upload_remote_directory} -name "${configuration_backup_prefix}*" | wc -l)
#count of old configuration backups
old_configuration_backup_count=$(find ${backup_directory}/${backup_upload_remote_directory} -name "${configuration_backup_prefix}*" -mtime +${configuration_backups_store_depth} -type f | wc -l)

if [ 0 -eq ${old_configuration_backup_count} ]
        then
                if [ ${configuration_backups_count} -lt ${configuration_backups_store_depth} ]
                        then echo "[`date +"${log_time_format}]: "`Attention!!! Storage contain ${configuration_backups_count} configuration backups, but storage depth is ${configuration_backups_store_depth}. Is it possible that backup task started later than ${configuration_backups_store_depth} days ago?" &>>${script_working_directory}/${log_file_name}
                        else echo "[`date +"${log_time_format}]: "`Attention!!! Storage contain ${configuration_backups_count} configuration backups, but all backups was created within storage depth interval (${configuration_backups_store_depth} days). Is it possible that in one day was created several backups (may be for testing purpose?)?" &>>${script_working_directory}/${log_file_name}
                fi
        else
                if [ ${configuration_backups_count} -gt ${configuration_backups_store_depth} ]
                        then
                                echo "[`date +"${log_time_format}]: "`Storage contain ${configuration_backups_count} configuration backups and ${old_configuration_backup_count} are older than ${configuration_backups_store_depth} days. Need to delete old files..." &>>${script_working_directory}/${log_file_name}
                                execute_command "Delete old configuration backups" "find ${backup_directory}/${backup_upload_remote_directory} -name "${configuration_backup_prefix}*" -mtime +${configuration_backups_store_depth} -type f -delete"
                        else echo "[`date +"${log_time_format}]: "`Attention!!! Outdated configuration backups are exist and new backups not enought! May be backup task work improper?" &>>${script_working_directory}/${log_file_name}
                fi
fi

#backup gitlab data with gitlab engine
echo "[`date +"${log_time_format}]: "`Starting GitLab backup engine..." &>>${script_working_directory}/${log_file_name}
execute_command "Backup GitLab data" "gitlab-rake gitlab:backup:create"
echo "[`date +"${log_time_format}]: "`GitLab backup engine has finished work" &>>${script_working_directory}/${log_file_name}

#dismount storage
execute_command "Dismount remote storage" "umount -l ${backup_directory}"

echo "[`date +"${log_time_format}]: "`Script has finished work, exit" &>>${script_working_directory}/${log_file_name}

echo "[`date +"${log_time_format}]: "`Sending e-mail report to ${admin_email}" &>>${script_working_directory}/${log_file_name}
mail -s "$(date "+%d/%m/%Y %H:%M:%S") - GitLab backup daily task" ${admin_email} <${script_working_directory}/${log_file_name}

#rm ${script_working_directory}/${log_file_name}